@@ -54,8 +54,8 @@ setValidity2("GRangesList", .valid.GRangesList)
 GRangesList <- function(...)
 {
     listData <- list(...)
-    if (length(listData) == 1L && is.list(listData[[1L]]))
-        listData <- listData[[1L]]
+    if (length(listData) == 1L && !is(listData[[1L]], "GRanges"))
+        return(as(listData[[1L]], "GRangesList"))
     if (length(listData) == 0L) {
         unlistData <- GRanges()
     } else {
@@ -158,7 +158,7 @@ setMethod("seqnames", "GRangesList",
 )
 
 ### NOT exported but used in GenomicAlignments package.
-replaceSeqnamesList <- function(x, value)
+set_GRangesList_seqnames <- function(x, value)
 {
     if (!is(value, "AtomicList") ||
         !identical(elementNROWS(x), elementNROWS(value)))
@@ -172,7 +172,7 @@ replaceSeqnamesList <- function(x, value)
     seqnames(x@unlistData) <- value
     x
 }
-setReplaceMethod("seqnames", "GRangesList", replaceSeqnamesList)
+setReplaceMethod("seqnames", "GRangesList", set_GRangesList_seqnames)
 
 setMethod("ranges", "GRangesList",
     function(x, use.names=TRUE, use.mcols=FALSE)
@@ -195,7 +195,7 @@ setMethod("ranges", "GRangesList",
 )
 
 setReplaceMethod("ranges", "GRangesList",
-    function(x, value) 
+    function(x, value)
     {
         if (!is(value, "RangesList") ||
             !identical(elementNROWS(x), elementNROWS(value)))
@@ -281,7 +281,7 @@ setMethod("strand", "GRangesList",
 )
 
 ### NOT exported but used in GenomicAlignments package.
-replaceStrandList <- function(x, value)
+set_GRangesList_strand <- function(x, value)
 {
     if (!is(value, "AtomicList") ||
         !identical(elementNROWS(x), elementNROWS(value)))
@@ -296,8 +296,8 @@ replaceStrandList <- function(x, value)
     strand(x@unlistData) <- value
     x
 }
-setReplaceMethod("strand", c("GRangesList", "ANY"), replaceStrandList)
-setReplaceMethod("strand", c("GRangesList", "character"), 
+setReplaceMethod("strand", c("GRangesList", "ANY"), set_GRangesList_strand)
+setReplaceMethod("strand", c("GRangesList", "character"),
     function(x, ..., value)
     {
         if (length(value) > 1L)
@@ -308,7 +308,7 @@ setReplaceMethod("strand", c("GRangesList", "character"),
 )
 
 ### NOT exported but used in GenomicAlignments package.
-getElementMetadataList <- 
+get_GRangesList_mcols <-
     function(x, use.names=FALSE, level = c("between", "within"), ...)
 {
     if (!isTRUEorFALSE(use.names))
@@ -326,10 +326,10 @@ getElementMetadataList <-
         rownames(unlisted_ans) <- names(unlisted_x)
     relist(unlisted_ans, x)
 }
-setMethod("elementMetadata", "GRangesList", getElementMetadataList)
+setMethod("elementMetadata", "GRangesList", get_GRangesList_mcols)
 
 ### NOT exported but used in GenomicAlignments package.
-replaceElementMetadataList <- 
+set_GRangesList_mcols <-
     function(x, level = c("between", "within"), ..., value)
 {
         level <- match.arg(level)
@@ -363,26 +363,42 @@ replaceElementMetadataList <-
     }
     x
 }
-setReplaceMethod("elementMetadata", "GRangesList", replaceElementMetadataList)
+setReplaceMethod("elementMetadata", "GRangesList", set_GRangesList_mcols)
 
 setMethod("seqinfo", "GRangesList", function(x) seqinfo(x@unlistData))
 
 ### NOT exported but used in GenomicAlignments package.
-replaceSeqinfoList <- function(x, new2old=NULL, force=FALSE, value)
+set_GRangesList_seqinfo <-
+    function(x, new2old=NULL, force=FALSE,
+             pruning.mode=c("error", "coarse", "fine", "tidy"),
+             value)
 {
+    pruning.mode <- match.arg(pruning.mode)
     if (!is(value, "Seqinfo"))
         stop("the supplied 'seqinfo' must be a Seqinfo object")
     dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
                               new2old=new2old, force=force,
+                              pruning.mode=pruning.mode,
                               seqlevels(value))
     if (length(dangling_seqlevels) != 0L) {
-        dropme <- which(seqnames(x@unlistData) %in% dangling_seqlevels)
-        x <- x[-unique(togroup(PartitioningByWidth(x), j=dropme))]
+        ## Prune 'x'.
+        non_dangling_range <- !(seqnames(x) %in% dangling_seqlevels)
+        if (pruning.mode == "coarse") {
+            x <- x[all(non_dangling_range)]
+        } else {
+            x <- x[non_dangling_range]  # "fine" pruning
+            if (pruning.mode == "tidy") {
+                ## Remove list elements that became empty because of "fine"
+                ## pruning.
+                x <- x[any(non_dangling_range) |
+                       elementNROWS(non_dangling_range) == 0L]
+            }
+        }
     }
     seqinfo(x@unlistData, new2old=new2old) <- value
     x
 }
-setReplaceMethod("seqinfo", "GRangesList", replaceSeqinfoList)
+setReplaceMethod("seqinfo", "GRangesList", set_GRangesList_seqinfo)
 
 setMethod("score", "GRangesList", function(x) {
   mcols(x)$score
@@ -410,9 +426,8 @@ setAs("GRangesList", "RangesList",
 
 setAs("GRanges", "GRangesList", function(from) as(from, "List"))
 
-setAs("RangedDataList", "GRangesList",
-      function(from) GRangesList(lapply(from, as, "GRanges")))
-
+setAs("list", "GRangesList",
+      function(from) do.call(GRangesList, from))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
@@ -504,6 +519,7 @@ setMethod("relistToClass", "GRanges", function(x) "GRangesList")
 ###
 
 ### NOT exported but used in GenomicAlignments package.
+### FIXME: This seems to repeat most of the code in IRanges:::showRangesList!
 showList <- function(object, showFunction, print.classinfo)
 {
     k <- length(object)
@@ -567,89 +583,6 @@ showList <- function(object, showFunction, print.classinfo)
 }
 
 setMethod("show", "GRangesList",
-    function(object)
-        showList(object, show_GenomicRanges, TRUE)
+    function(object) showList(object, show_GenomicRanges, TRUE)
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Deconstruction/reconstruction of a GRangesList into/from a GRanges
-### object.
-###
-### For internal use only (not exported).
-###
-
-### Unlist GRangesList object 'x' into a GRanges object but the differences
-### with the "unlist" method for GRangesList objects are:
-###   - The sequence names of the returned GRanges object are modified by
-###     embedding the "grouping by top-level element" information in them.
-###   - The seqinfo is modified accordingly.
-deconstructGRLintoGR <- function(x, expand.levels=FALSE)
-{
-    ans <- x@unlistData
-    f1 <- rep.int(seq_len(length(x)), elementNROWS(x))
-    f2 <- as.integer(seqnames(ans))
-    f12 <- paste(f1, f2, sep="|")
-
-    ## Compute 'ans_seqinfo'.
-    if (expand.levels) {
-        x_nlev <- length(seqlevels(x))
-        i1 <- rep(seq_len(length(x)), each=x_nlev)
-        i2 <- rep.int(seq_len(x_nlev), length(x))
-    } else {
-        oo <- orderIntegerPairs(f1, f2)
-        of1 <- f1[oo]
-        of2 <- f2[oo]
-        ## TODO: Support 'method="presorted"' in duplicatedIntegerPairs() for
-        ## when the 2 input vectors are already sorted.
-        notdups <- !duplicatedIntegerPairs(of1, of2)
-        i1 <- of1[notdups]
-        i2 <- of2[notdups]
-    }
-    x_seqinfo <- seqinfo(x)
-    ans_seqlevels <- paste(i1, i2, sep="|")
-    ans_seqlengths <- unname(seqlengths(x_seqinfo))[i2]
-    ans_isCircular <- unname(isCircular(x_seqinfo))[i2]
-    ans_seqinfo <- Seqinfo(ans_seqlevels, ans_seqlengths, ans_isCircular)
-
-    ## The 2 following modifications must be seen as a single atomic
-    ## operation since doing the 1st without doing the 2nd would leave 'ans'
-    ## in a broken state.
-    ans@seqnames <- Rle(factor(f12, ans_seqlevels))
-    ans@seqinfo <- ans_seqinfo
-    ans
-}
-
-### The "inverse" transform of deconstructGRLintoGR().
-### More precisely, reconstructGRLfromGR() transforms GRanges object 'gr'
-### with sequence names in the "f1|f2" format (as produced by
-### deconstructGRLintoGR() above) back into a GRangesList object with the
-### same length & names & metadata columns & seqinfo as 'x'.
-### The fundamental property of this deconstruction/reconstruction mechanism
-### is that, for any GRangesList object 'x':
-###
-###   reconstructGRLfromGR(deconstructGRLintoGR(x), x) is identical to x
-###
-reconstructGRLfromGR <- function(gr, x)
-{
-    snames <- strsplit(as.character(seqnames(gr)), "|", fixed=TRUE)
-    m12 <- matrix(as.integer(unlist(snames)), ncol=2, byrow=TRUE)
-
-    ## Restore the real sequence names.
-    f2 <- m12[ , 2L]
-    x_seqlevels <- seqlevels(x)
-    ## The 2 following modifications must be seen as a single atomic
-    ## operation since doing the 1st without doing the 2nd would leave 'ans'
-    ## in a broken state.
-    gr@seqnames <- Rle(factor(x_seqlevels[f2], x_seqlevels))
-    gr@seqinfo <- seqinfo(x)
-
-    ## Split.
-    f1 <- m12[ , 1L]
-    ans <- split(gr, factor(f1, levels=seq_len(length(x))))
-    names(ans) <- names(x)
-    metadata(ans) <- metadata(x)
-    mcols(ans) <- mcols(x)
-    ans
-}
-
