@@ -10,7 +10,7 @@ setClass("GenomicRanges",
     representation(
         "VIRTUAL"#,
         #No more constraint slot for now...
-        #constraint="ConstraintORNULL"
+        #constraint="Constraint_OR_NULL"
     )
 )
 
@@ -390,30 +390,42 @@ setReplaceMethod("strand", "GenomicRanges",
     }
 )
 
-setReplaceMethod("seqinfo", "GenomicRanges",
-    function(x, new2old=NULL, force=FALSE, value)
-    {
-        if (!is(value, "Seqinfo"))
-            stop("the supplied 'seqinfo' must be a Seqinfo object")
-        dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
-                                  new2old=new2old, force=force,
-                                  seqlevels(value))
-        if (length(dangling_seqlevels) != 0L)
-            x <- x[!(seqnames(x) %in% dangling_seqlevels)]
-        old_seqinfo <- seqinfo(x)
-        new_seqnames <- GenomeInfoDb:::makeNewSeqnames(x,
-                                  new2old=new2old, seqlevels(value))
-        x <- update(x, seqnames=new_seqnames, seqinfo=value, check=FALSE)
-        geom_has_changed <- GenomeInfoDb:::sequenceGeometryHasChanged(
-                                  seqinfo(x), old_seqinfo, new2old=new2old)
-        if (any(geom_has_changed, na.rm=TRUE)) {
-            msg <- valid.GenomicRanges.seqinfo(x, suggest.trim=TRUE)
-            if (!is.null(msg))
-                stop(msg)
-        }
-        x
+### Does NOT suppoprt pruning mode "fine". Pruning modes "coarse" and "tidy"
+### are equivalent on a GenomicRanges object.
+set_GenomicRanges_seqinfo <-
+    function(x, new2old=NULL, force=FALSE,
+             pruning.mode=c("error", "coarse", "fine", "tidy"),
+             value)
+{
+    pruning.mode <- match.arg(pruning.mode)
+    if (pruning.mode == "fine")
+        stop(wmsg("\"fine\" pruning mode not supported on ",
+                  class(x), " objects"))
+    if (!is(value, "Seqinfo"))
+        stop("the supplied 'seqinfo' must be a Seqinfo object")
+    dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
+                              new2old=new2old, force=force,
+                              pruning.mode=pruning.mode,
+                              seqlevels(value))
+    if (length(dangling_seqlevels) != 0L) {
+        ## Prune 'x'.
+        non_dangling_range <- !(seqnames(x) %in% dangling_seqlevels)
+        x <- x[non_dangling_range]
     }
-)
+    old_seqinfo <- seqinfo(x)
+    new_seqnames <- GenomeInfoDb:::makeNewSeqnames(x,
+                              new2old=new2old, seqlevels(value))
+    x <- update(x, seqnames=new_seqnames, seqinfo=value, check=FALSE)
+    geom_has_changed <- GenomeInfoDb:::sequenceGeometryHasChanged(
+                              seqinfo(x), old_seqinfo, new2old=new2old)
+    if (any(geom_has_changed, na.rm=TRUE)) {
+        msg <- valid.GenomicRanges.seqinfo(x, suggest.trim=TRUE)
+        if (!is.null(msg))
+            stop(msg)
+    }
+    x
+}
+setReplaceMethod("seqinfo", "GenomicRanges", set_GenomicRanges_seqinfo)
 
 setMethod("score", "GenomicRanges", function(x) mcols(x)$score)
 setReplaceMethod("score", "GenomicRanges", function(x, value) {
@@ -426,7 +438,7 @@ setReplaceMethod("score", "GenomicRanges", function(x, value) {
 #    {
 #        if (isSingleString(value))
 #            value <- new(value)
-#        if (!is(value, "ConstraintORNULL"))
+#        if (!is(value, "Constraint_OR_NULL"))
 #            stop("the supplied 'constraint' must be a ",
 #                 "Constraint object, a single string, or NULL")
 #        x@constraint <- value
@@ -604,7 +616,7 @@ setMethod("[", c("list", "GenomicRanges"),
 ### instead of 'x$name'.
 ###
 
-.DollarNames.GenomicRanges <- function(x, pattern)
+.DollarNames.GenomicRanges <- function(x, pattern = "")
     grep(pattern, names(mcols(x)), value=TRUE)
 
 setMethod("$", "GenomicRanges",
