@@ -19,8 +19,8 @@
 ### implicitly. Also, note that, because we already do (c) and (d) for regular
 ### ranges, genomic ranges that belong to the same underlying sequence
 ### and strand are ordered like regular ranges.
-### order(), sort(), and rank() on a GenomicRanges object are using this
-### "natural order".
+### is.unsorted(), order(), sort(), and rank() on a GenomicRanges object
+### adhere to this "natural order".
 ###
 ### III. ELEMENT-WISE (AKA "PARALLEL") COMPARISON OF 2 GenomicRanges OBJECTS
 ### ------------------------------------------------------------------------
@@ -93,6 +93,7 @@ setMethod("pcompare", c("GenomicRanges", "GenomicRanges"),
 ###
 
 .duplicated.GenomicRanges <- function(x, incomparables=FALSE, fromLast=FALSE,
+                                      nmax = NA,
                                       method=c("auto", "quick", "hash"))
 {
     if (!identical(incomparables, FALSE))
@@ -232,7 +233,8 @@ setMethod("is.unsorted", "GenomicRanges",
     }
 )
 
-.order_GenomicRanges <- function(x, decreasing=FALSE, ignore.strand=FALSE)
+### NOT exported but used in GenomicAlignments package.
+order_GenomicRanges <- function(x, decreasing=FALSE, ignore.strand=FALSE)
 {
     if (!isTRUEorFALSE(decreasing))
         stop("'decreasing' must be TRUE or FALSE")
@@ -248,7 +250,8 @@ setMethod("is.unsorted", "GenomicRanges",
 ### ignored.
 ### 'method' is also ignored at the moment.
 setMethod("order", "GenomicRanges",
-    function(..., na.last=TRUE, decreasing=FALSE, method=c("shell", "radix"))
+    function(..., na.last=TRUE, decreasing=FALSE,
+                  method=c("auto", "shell", "radix"))
     {
         ## Turn off this warning for now since it triggers spurious warnings
         ## when calling sort() on a GRangesList object. The root of the
@@ -263,7 +266,7 @@ setMethod("order", "GenomicRanges",
         ## All arguments in '...' are guaranteed to be GenomicRanges objects.
         args <- list(...)
         if (length(args) == 1L)
-            return(.order_GenomicRanges(args[[1L]], decreasing))
+            return(order_GenomicRanges(args[[1L]], decreasing))
         order_args <- c(unlist(lapply(args, .GenomicRanges_as_IntegerQuads),
                                recursive=FALSE, use.names=FALSE),
                         list(na.last=na.last, decreasing=decreasing))
@@ -275,7 +278,7 @@ setMethod("order", "GenomicRanges",
 .sort.GenomicRanges <- function(x, decreasing=FALSE, ignore.strand=FALSE, by)
 {
     if (missing(by)) {
-        oo <- .order_GenomicRanges(x, decreasing, ignore.strand)
+        oo <- order_GenomicRanges(x, decreasing, ignore.strand)
     } else {
         if (!identical(ignore.strand, FALSE))
             warning("'ignore.strand' ignored when 'by' is specified")
@@ -287,3 +290,16 @@ sort.GenomicRanges <- function(x, decreasing=FALSE, ...)
     .sort.GenomicRanges(x, decreasing=decreasing, ...)
 setMethod("sort", "GenomicRanges", .sort.GenomicRanges)
 
+setMethod("rank", "GenomicRanges",
+    function(x, na.last=TRUE,
+             ties.method=c("average", "first", "last", "random", "max", "min"),
+             ignore.strand=FALSE)
+    {
+        if (!isTRUEorFALSE(ignore.strand))
+            stop("'ignore.strand' must be TRUE of FALSE")
+        if (ignore.strand)
+            x <- unstrand(x)
+        callNextMethod(x, na.last=na.last, ties.method=ties.method)
+    }
+)
+
